@@ -100,6 +100,12 @@ static int check_min_rate(struct thread_data *td, struct timeval *now)
 	unsigned long spent;
 	unsigned long rate;
 
+	/*
+	 * No minimum rate set, always ok
+	 */
+	if (!td->ratemin)
+		return 0;
+
 	/*
 	 * allow a 2 second settle period in the beginning
 	 */
@@ -237,7 +243,7 @@ requeue:
 }
 
 /*
- * The main verify engine. Runs over the writes we previusly submitted,
+ * The main verify engine. Runs over the writes we previously submitted,
  * reads the blocks back in, and checks the crc/md5 of the data.
  */
 static void do_verify(struct thread_data *td)
