@@ -34,7 +34,7 @@ enum td_ddir {
 	TD_DDIR_RAND		= 1 << 2,
 	TD_DDIR_RW		= TD_DDIR_READ | TD_DDIR_WRITE,
 	TD_DDIR_RANDREAD	= TD_DDIR_READ | TD_DDIR_RAND,
-	TD_DDIR_RANDWRITE	= TD_DDIR_WRITE | TD_DDIR_READ,
+	TD_DDIR_RANDWRITE	= TD_DDIR_WRITE | TD_DDIR_RAND,
 	TD_DDIR_RANDRW		= TD_DDIR_RW | TD_DDIR_RAND,
 };
 
@@ -214,8 +214,8 @@ enum fio_ioengine_flags {
 	FIO_CPUIO	= 1 << 1,	/* cpu burner, doesn't do real io */
 	FIO_MMAPIO	= 1 << 2,	/* uses memory mapped io */
 	FIO_RAWIO	= 1 << 3,	/* some sort of direct/raw io */
-	FIO_NETIO	= 1 << 4,	/* networked io */
-	FIO_NULLIO	= 1 << 5,	/* no real data transfer (cpu/null) */
+	FIO_DISKLESSIO	= 1 << 4,	/* no disk involved */
+	FIO_SELFOPEN	= 1 << 5,       /* opens its own devices */
 };
 
 /*
@@ -288,7 +288,7 @@ struct thread_data {
 	char *name;
 	char *directory;
 	char *filename;
-	char verror[80];
+	char verror[128];
 	pthread_t thread;
 	int thread_number;
 	int groupid;
@@ -296,6 +296,7 @@ struct thread_data {
 	enum fio_filetype filetype;
 	struct fio_file *files;
 	unsigned int nr_files;
+	unsigned int nr_open_files;
 	unsigned int nr_uniq_files;
 	union {
 		unsigned int next_file;
@@ -618,6 +619,8 @@ extern int __must_check init_random_state(struct thread_data *);
 extern void close_files(struct thread_data *);
 extern int __must_check setup_files(struct thread_data *);
 extern int __must_check open_files(struct thread_data *);
+extern int open_file(struct thread_data *, struct fio_file *, int, int);
+extern void close_file(struct thread_data *, struct fio_file *);
 extern int __must_check file_invalidate_cache(struct thread_data *, struct fio_file *);
 
 /*
@@ -715,6 +718,9 @@ static inline void fio_sem_up(volatile int *sem)
 		fprintf(stderr, ##args);	\
 	} while (0)
 
+FILE *get_f_out(void);
+FILE *get_f_err(void);
+
 struct ioengine_ops {
 	struct list_head list;
 	char name[16];
