@@ -103,6 +103,7 @@ static struct fio_option options[] = {
 #ifdef FIO_HAVE_SYSLET
 			  { .ival = "syslet-rw", },
 #endif
+			  { .ival = "external", },
 			  },
 	},
 	{
@@ -394,7 +395,7 @@ static struct fio_option options[] = {
 		.name	= "thinktime_spin",
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(thinktime_spin),
-		.help	= "Start thinktime by spinning this amount (usec)",
+		.help	= "Start think time by spinning this amount (usec)",
 		.def	= "0",
 	},
 	{
@@ -580,7 +581,7 @@ static struct option long_options[FIO_JOB_OPTS + FIO_CMD_OPTS] = {
 	},
 	{
 		.name		= "cmdhelp",
-		.has_arg	= required_argument,
+		.has_arg	= optional_argument,
 		.val		= 'c',
 	},
 	{
@@ -590,7 +591,7 @@ static struct option long_options[FIO_JOB_OPTS + FIO_CMD_OPTS] = {
 
 static int def_timeout = 0;
 
-static char fio_version_string[] = "fio 1.12";
+static char fio_version_string[] = "fio 1.13";
 
 static char **ini_file;
 static int max_jobs = MAX_JOBS;
@@ -607,6 +608,16 @@ FILE *f_err = NULL;
 static int write_lat_log = 0;
 int write_bw_log = 0;
 
+FILE *get_f_out()
+{
+	return f_out;
+}
+
+FILE *get_f_err()
+{
+	return f_err;
+}
+
 /*
  * Return a free job structure.
  */
@@ -1017,11 +1028,26 @@ static int str_mem_cb(void *data, const char *mem)
 	return 0;
 }
 
+/* External engines are specified by "external:name.o") */
+static const char *get_engine_name(const char *str)
+{
+	char *p = strstr(str, ":");
+
+	if (!p)
+		return str;
+
+	p++;
+	strip_blank_front(&p);
+	strip_blank_end(p);
+	return p;
+}
+
 static int str_ioengine_cb(void *data, const char *str)
 {
 	struct thread_data *td = data;
+	const char *name = get_engine_name(str);
 
-	td->io_ops = load_ioengine(td, str);
+	td->io_ops = load_ioengine(td, name);
 	if (td->io_ops)
 		return 0;
 
@@ -1197,7 +1223,7 @@ static int parse_cmd_line(int argc, char *argv[])
 	struct thread_data *td = NULL;
 	int c, ini_idx = 0, lidx, ret, dont_add_job = 0;
 
-	while ((c = getopt_long(argc, argv, "", long_options, &lidx)) != -1) {
+	while ((c = getopt_long_only(argc, argv, "", long_options, &lidx)) != -1) {
 		switch (c) {
 		case 't':
 			def_timeout = atoi(optarg);
