@@ -14,7 +14,7 @@ static struct list_head disk_list = LIST_HEAD_INIT(disk_list);
 static dev_t last_dev;
 
 /*
- * Cheasy number->string conversion, complete with carry rounding error.
+ * Cheesy number->string conversion, complete with carry rounding error.
  */
 static char *num2str(unsigned long num, int maxlen, int base)
 {
@@ -246,7 +246,7 @@ void init_disk_util(struct thread_data *td)
 	dev_t dev;
 	char *p;
 
-	if (!td->do_disk_util || (td->io_ops->flags & (FIO_NETIO | FIO_NULLIO)))
+	if (!td->do_disk_util || (td->io_ops->flags & FIO_DISKLESSIO))
 		return;
 
 	/*
