@@ -51,7 +51,7 @@ static int verify_io_u_crc32(struct verify_header *hdr, struct io_u *io_u)
 	c = crc32(p, hdr->len - sizeof(*hdr));
 
 	if (c != hdr->crc32) {
-		log_err("crc32: verify failed at %llu/%u\n", io_u->offset, io_u->buflen);
+		log_err("crc32: verify failed at %llu/%lu\n", io_u->offset, io_u->buflen);
 		log_err("crc32: wanted %lx, got %lx\n", hdr->crc32, c);
 		return 1;
 	}
@@ -69,7 +69,7 @@ static int verify_io_u_md5(struct verify_header *hdr, struct io_u *io_u)
 	md5_update(&md5_ctx, p, hdr->len - sizeof(*hdr));
 
 	if (memcmp(hdr->md5_digest, md5_ctx.hash, sizeof(md5_ctx.hash))) {
-		log_err("md5: verify failed at %llu/%u\n", io_u->offset, io_u->buflen);
+		log_err("md5: verify failed at %llu/%lu\n", io_u->offset, io_u->buflen);
 		hexdump(hdr->md5_digest, sizeof(hdr->md5_digest));
 		hexdump(md5_ctx.hash, sizeof(md5_ctx.hash));
 		return 1;
@@ -78,13 +78,15 @@ static int verify_io_u_md5(struct verify_header *hdr, struct io_u *io_u)
 	return 0;
 }
 
-static int verify_io_u(struct io_u *io_u)
+int verify_io_u(struct io_u *io_u)
 {
 	struct verify_header *hdr = (struct verify_header *) io_u->buf;
 	int ret;
 
-	if (hdr->fio_magic != FIO_HDR_MAGIC)
+	if (hdr->fio_magic != FIO_HDR_MAGIC) {
+		log_err("Bad verify header %x\n", hdr->fio_magic);
 		return 1;
+	}
 
 	if (hdr->verify_type == VERIFY_MD5)
 		ret = verify_io_u_md5(hdr, io_u);
@@ -141,6 +143,12 @@ int get_next_verify(struct thread_data *td, struct io_u *io_u)
 {
 	struct io_piece *ipo;
 
+	/*
+	 * this io_u is from a requeue, we already filled the offsets
+	 */
+	if (io_u->file)
+		return 0;
+
 	if (!list_empty(&td->io_hist_list)) {
 		ipo = list_entry(td->io_hist_list.next, struct io_piece, list);
 
@@ -148,24 +156,13 @@ int get_next_verify(struct thread_data *td, struct io_u *io_u)
 
 		io_u->offset = ipo->offset;
 		io_u->buflen = ipo->len;
+		io_u->file = ipo->file;
 		io_u->ddir = DDIR_READ;
+		io_u->xfer_buf = io_u->buf;
+		io_u->xfer_buflen = io_u->buflen;
 		free(ipo);
 		return 0;
 	}
 
 	return 1;
 }
-
-int do_io_u_verify(struct thread_data *td, struct io_u **io_u)
-{
-	struct io_u *v_io_u = *io_u;
-	int ret = 0;
-
-	if (v_io_u) {
-		ret = verify_io_u(v_io_u);
-		put_io_u(td, v_io_u);
-		*io_u = NULL;
-	}
-
-	return ret;
-}
